package com.steerinfo.dil.service.impl;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.steerinfo.dil.model.TmsshipTotalResult;
import com.steerinfo.dil.util.DataChange;
import com.steerinfo.framework.mapper.IBaseMapper;
import com.steerinfo.framework.service.impl.BaseServiceImpl;
import com.steerinfo.dil.model.TmsshipShipLocation;
import com.steerinfo.dil.mapper.TmsshipShipLocationMapper;
import com.steerinfo.dil.service.ITmsshipShipLocationService;
import com.steerinfo.framework.utils.base.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * TmsshipShipLocation服务实现：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-08-19 08:52
 * 类描述
 * 修订历史：
 * 日期：2021-08-19
 * 作者：generator
 * 参考：
 * 描述：TmsshipShipLocation服务实现
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@Service(value = "tmsshipShipLocationService")
public class TmsshipShipLocationServiceImpl implements ITmsshipShipLocationService {

    @Autowired
    TmsshipShipLocationMapper tmsshipShipLocationMapper;
    @Autowired
    TmsshipTotalResultServiceImpl tmsshipTotalResultService;



    @Override
    public BigDecimal selectMaxId() {
        return tmsshipShipLocationMapper.selectLocationId();
    }


    /**
     * 删除位置作业
     * @param locationId
     * @return
     */
    @Override
    public int deleteShipLocation(BigDecimal locationId) {
        TmsshipShipLocation tmsshipShipLocation = tmsshipShipLocationMapper.selectByPrimaryKey(locationId);
        tmsshipShipLocation.setDeleted(new BigDecimal(1));
        return tmsshipShipLocationMapper.updateByPrimaryKeySelective(tmsshipShipLocation);
    }

    /**
     * 添加位置作业
     * @param map
     * @return
     */
    @Override
    public int addShipLocation(Map<String,Object> map) {
        Map<String,Object> mapVal = (Map<String, Object>) map.get("mapVal");
        Map<String,Object> shipLocationMap = (Map<String, Object>) map.get("shipLocationMap");
        BigDecimal orderId = new BigDecimal((Integer) shipLocationMap.get("orderId"));
        BigDecimal totalResultId = tmsshipTotalResultService.getResultTotalId(orderId);
        String locationValue = (String) mapVal.get("locationValue");
        Long locationRouteTimeLong = (Long) mapVal.get("locationRouteTime");
        Date locationRouteTime = new Date(locationRouteTimeLong);
        String locationMemo = (String)mapVal.get("locationMemo");
        String locationStatus = (String)mapVal.get("locationStatus");
        BigDecimal locationId = selectMaxId();
        TmsshipShipLocation tmsshipShipLocation = new TmsshipShipLocation();
        tmsshipShipLocation.setLocationId(locationId);
        tmsshipShipLocation.setTotalResultId(totalResultId);
        tmsshipShipLocation.setLocationValue(locationValue);
        tmsshipShipLocation.setLocationRouteTime(locationRouteTime);
        tmsshipShipLocation.setLocationMemo(locationMemo);
        tmsshipShipLocation.setLocationStatus(locationStatus);
        // 设置常规字段
        tmsshipShipLocation.setDeleted(new BigDecimal(0));
        tmsshipShipLocation.setInsertTime(new Date());
        tmsshipShipLocation.setInsertUsername("admin");
        tmsshipShipLocation.setInsertUpdateRemark("无");
        tmsshipShipLocation.setDeleteName(null);
        tmsshipShipLocation.setDeleteTime(null);
        tmsshipShipLocation.setUpdateTime(new Date());
        tmsshipShipLocation.setUpdateUsername("admin");
        return tmsshipShipLocationMapper.insert(tmsshipShipLocation);
    }

    /**
     * 查询位置作业详情
     * @param locationId
     * @return
     */
    @Override
    public TmsshipShipLocation selectShipLocation(BigDecimal locationId) {
        return tmsshipShipLocationMapper.selectByPrimaryKey(locationId);
    }

    /**
     * 修改位置作业
     * @param map
     * @return
     */
    @Override
    public int updateShipLocation(Map<String, Object> map) {
        TmsshipShipLocation tmsshipShipLocation = new TmsshipShipLocation();
        //获取主键
        tmsshipShipLocation.setLocationId(DataChange.dataToBigDecimal(map.get("locationId")));
        //获取船舶位置
        tmsshipShipLocation.setLocationValue((String) map.get("shipLocation"));
        //获取修改船舶日期
        tmsshipShipLocation.setLocationRouteTime(new Date());
        //船舶位置状态
        tmsshipShipLocation.setLocationStatus((String) map.get("locationStatus"));
        tmsshipShipLocation.setUpdateTime(new Date());
        return tmsshipShipLocationMapper.updateByPrimaryKeySelective(tmsshipShipLocation);
    }

    /**
     * 查看所有位置作业信息
     * @return
     */
    @Override
    public List<Map<String, Object>> selectShipLocationList(Map<String,Object> mapVal) {
        return tmsshipShipLocationMapper.selectShipLocationList(mapVal);
    }

    /**
     * 查看位置作业表单数据
     * @param locationId
     * @return
     */
    @Override
    public List<Map<String, Object>> getShipLocation(BigDecimal locationId) {
        return tmsshipShipLocationMapper.selectShipLocation(locationId);
    }

    @Override
    public Map<String, Object> getShipLocationStatus(String shipLocation) {
        return tmsshipShipLocationMapper.getShipLocationStatus(shipLocation);
    }

    @Override
    public List<Map<String, Object>> selectShipLocationWarningList() {
        return tmsshipShipLocationMapper.selectShipLocationWarningList();
    }

    @Override
    public int batchInsertShipWarning(List<Map<String, Object>> list) {
        return tmsshipShipLocationMapper.batchInsertShipWarning(list);
    }

    @Override
    public List<Map<String, Object>> selectShipWarningList(Map<String, Object> mapVal) {
        return tmsshipShipLocationMapper.selectShipWarningList(mapVal);
    }

    @Override
    public int updateShipWarning(Map<String, Object> mapVal) {
        mapVal.put("insertUserName", "");
        if(mapVal.get("userId") != null && !mapVal.get("userId").toString().trim().isEmpty()){
            mapVal.put("insertUserName", tmsshipShipLocationMapper.getUserInfoById(mapVal));
        }
        return tmsshipShipLocationMapper.updateShipWarning(mapVal);
    }

    @Scheduled(fixedRate = 1000*60*60*12)
    public void createShipWarning(){
        try {
            System.out.println("开始生成船舶预警"+ DateUtils.getCommonDateStr(new Date()));
            List<Map<String, Object>> columnList = tmsshipShipLocationMapper.selectShipLocationWarningList();
            List<Map<String, Object>> list = new ArrayList<>();
            if(columnList != null && !columnList.isEmpty()) {
                for (Map<String, Object> map : columnList) {
                    String locationRouteTime = map.get("locationRouteTime").toString();
                    Map<String, Object> insertMap = new HashMap<String, Object>();
                    if(locationRouteTime != null && !locationRouteTime.isEmpty()){
                        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                        Date locationRouteTimeDate = sdf.parse(locationRouteTime);
                        long starTime=locationRouteTimeDate.getTime();
                        long endTime=(new Date()).getTime();
                        long num=endTime-starTime;//时间戳相差的毫秒数
                        //超过15天提醒
                        if((num/(24*60*60*1000)) > 15){
                            insertMap.put("abnormalContent", map.get("materialName") +" \""+ map.get("foreignShipName") +"\" "+map.get("capacityName")
                                    +"，离港日期"+ map.get("locationRouteTime") +"，离港已超过十五天。");
                            insertMap.put("abnormalType", "离港超时");
                            insertMap.put("abnormalTime", new Date());
                            insertMap.put("abnormalStatus", "未处理");
                            insertMap.put("orderId", map.get("orderId"));
                            list.add(insertMap);
                        }
                    }
                }
            }
            if(!list.isEmpty()){
                tmsshipShipLocationMapper.batchInsertShipWarning(list);
            }
        } catch (Exception e) {
            System.out.println("生成船舶预警出错：" + e.getMessage());
        }
    }
}
